
{*******************************************************}
{                                                       }
{       AnimatedMenus/2000                              }
{       T_AM2000_MenuItemCache Unit                     }
{       English Version                                 }
{                                                       }
{       Copyright (c) 1997-99 AnimatedMenus.com         }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000cache;

interface

uses
  Windows, CommCtrl, Classes, Graphics;

const
  // am2000designer
  SBrowseForMenuTemplateFolder =
    'Please select a folder for AnimatedMenus/2000 templates.';
  SInsertMenuStart =
    'Insert ';
  SInsertMenuFinish =
    ' Menu';
  SBitmapDialogFilter =
    'Bitmaps (*.bmp)|*.bmp|All Files (*.*)|*.*';
  SSaveBitmapDialogTitle =
    'Save Bitmap As';
  SOpenBitmapDialogTitle =
    'Open Bitmap';
  SDefaultMenuItemCaption =
    'Menu Item';

  // am2000utils
  SDoesntSupportTitle =
    'AnimatedMenus/2000 2.1 Evaluation Edition';
  SDoesntSupportText1 =
    'Sorry, but AnimatedMenus/2000 Evaluation Edition doesn''t support ';
  SDoesntSupportText2 = '. '+
    'Please upgrade your version to Standard or Developer Edition.'#13#13 +
    'Visit www.animatedmenus.com for more information.'#13#13 +
    'Thank you for evaluating AnimatedMenus/2000!'#13#13 +
    'Sincerely, Andrew Cher,'#13'AnimatedMenus.com, Inc.';
  SThisFeature =
    'this feature';
  SSubmenus =
    'submenus';
  SFileOperations =
    'file operations';
  SFloatingMenus =
    'floating menus';
  SMergeUnmerge =
    'multiple menu merging';
  SClipboardOperations =
    'clipboard operations';


  // am2000cache
  SShift = 'Shift';
  SCtrl = 'Ctrl';
  SAlt = 'Alt';
  SIns = 'Ins';
  SDel = 'Del';

  // am2000mainmenu
  SMenuIndexOutOfBounds =
    'Menu index is out of bounds';

  // am2000menuitem
  SInvalidShortCut =
    'Invalid shortcut: ''';
  SThisIsNotAButtonArray =
    ': This is not a Button Array';
  SThisIsNotAButton =
    ': This is not a Button';
  SThisIsNotABitmap =
    ': This is not a Bitmap';
  SThisisNotAGif =
    ': This is not a Gif image';
  SThisIsNotAEdit =
    ': This is not an Editbox';
  SThisIsNotACombo =
    ': This is not an Combobox';
  SThisIsNotAList =
    ': This is not a Listbox';
  SMenuNotFound =
    'Menu not found';
  SNoNewMenu =
    'You cannot use NewMenu function with AnimatedMenus/2000';
  SNoNewPopupMenu =
    'You cannot use NewPopupMenu function with AnimatedMenus/200';

  // am2000popup
  SDraggableMenuInfo =
    'Drag to make this menu float';
  SExpand =
    'Expand (Ctrl+Down)';

  // am2000
  SWelcomeMessage = 'Hi!'#13#13+
    'Thank you for using AnimatedMenus/2000 Evaluation '+
    'Edition. We hope you''ll enjoy this software. Visit our website at'+
    'http://www.animatedmenus.com/ for the documentation, samples, '+
    'knwoledge base, latest software updates, and gifts for users.'#13#13+
    'Thank you!'#13#13+
    'Sincerely, Andrew Cher,'#13+
    'AnimatedMenus.com, Inc.'#13#13+
    'P.S. This is one-time showing message, you''ll not see this message again.';

  // am2000bitmap
  SValueMustBeBetween1And4 =
    'Value must be between 1 and 4.';

  // am2000editbox
  SOnlyTextMenuItemCanBeEdited =
    'Only text menu item can be edited.';

type
  T_AM2000_MenuItemCacheItem = class
  public
    ShortCuts : String;
    Hint      : String;
    IsDefault : Boolean;
    HasBitmap : Boolean;
  end;

  T_AM2000_MenuItemCache = class(TStringList)
  private
    Images: HImageList;

    function GetItem(const Caption: String): T_AM2000_MenuItemCacheItem;

  public
    property Items    [const Caption: String]: T_AM2000_MenuItemCacheItem read GetItem; default;

    constructor Create;
    destructor Destroy; override;
    function AddItem(ACaption, AShortCuts, AHint: String; ABitmap, ADefault: Boolean): T_AM2000_MenuItemCacheItem;

    function IsDefault(Caption: String): Boolean;
    procedure Draw(Canvas: TCanvas; X, Y: Integer; Caption: String; Color1, Color2: TColor; Enabled: Boolean);
    procedure DrawIndex(Canvas: TCanvas; X, Y, ImageIndex: Integer; Color1, Color2: TColor; Enabled: Boolean);

  end;


var
  MenuItemCache: T_AM2000_MenuItemCache;

implementation

uses
  SysUtils,
  am2000utils;

{ Utilities }

function GetStandardCaption(S: String): String;
var
  J: Integer;
begin
  Result:= '';
  S:= AnsiUpperCase(S);
  for J:= 1 to Length(S) do
    if IsCharAlpha(S[J])
    then AppendStr(Result, S[J]);
end;


{ T_AM2000_MenuItemCache }

constructor T_AM2000_MenuItemCache.Create;
begin
  inherited;
  Sorted:= True;

  // initialize default menu items
  AddItem('ADDRESSBOOK', '', 'Displays the list of contacts', True, True);
  AddItem('ALIGNCENTER', SCtrl+'+E', 'Centers the paragraph between the margins', True, True);
  AddItem('ALIGNLEFT', SCtrl+'+L', 'Aligns the paragraph at the left margin', True, True);
  AddItem('ALIGNRIGHT', SCtrl+'+R', 'Aligns the paragraph at the right margin', True, True);
  AddItem('AUTOPREVIEW', '', 'Shows or hides the autopreview pane', True, True);
  AddItem('BOLD', SCtrl+'+B', 'Bold Text', True, True);
  AddItem('BOOKMARKS', '', 'Insert a bookmark at the cursor', True, True);
  AddItem('BULLETS', '', 'Inserts a bullet on this line', True, True);
  AddItem('CASCADE', '', 'Arranges the windows as overlapping tiles', True, True);
  AddItem('CONTENTS', '', 'Displays help topics', True, False);
  AddItem('CONTENTSANDINDEX', 'F1', 'Displays help topics', True, True);
  AddItem('COPY', SCtrl+'+C;'+SCtrl+'+'+SIns, 'Copies the selection to the clipboard', True, True);
  AddItem('CUT', SCtrl+'+X;'+SShift+'+'+SDel, 'Cuts the selection and moves it to the clipboard', True, True);
  AddItem('DATE', '', 'Inserts today''s date', True, False);
  AddItem('DATEANDTIME', '', 'Inserts today''s date and/or time', True, True);
  AddItem('DATETIME', '', 'Inserts today''s date and/or time', True, False);
  AddItem('DECREASEINDENT', '', 'Decrease indent', True, True);
  AddItem('DELETE', ''+SDel+';'+SCtrl+'+'+SDel, 'Erases the selection', True, True);
  AddItem('FIND', SCtrl+'+F;F3', 'Finds the specified text in the active document', True, True);
  AddItem('FINDNEXT', SShift+'+F3', 'Repeats the last find', True, True);
  AddItem('FONT', '', 'View or edit the character attributes of the selected text', True, False);
  AddItem('FONTSIZE', '', 'View or edit the font size of the selected text', True, True);
  AddItem('FONTSTYLE', '', 'View or edit the font style of the selected text', True, True);
  AddItem('FULLSCREEN', 'F11', 'Zooms the active document to the full screen', True, True);
  AddItem('GOTOBOOKMARK', '', 'Moves the cursor to the specified bookmark', True, False);
  AddItem('HELPONHELP', 'F1', 'Display instructions about how to use help', True, True);
  AddItem('HOME', '', 'Browses the web', True, True);
  AddItem('HYPERLINK', '', 'Insert a new hyperlink', True, True);
  AddItem('INCREASEINDENT', '', 'Increase indent', True, True);
  AddItem('INDENT', '', 'Increase Indent', True, False);
  AddItem('ITALIC', SCtrl+'+I', 'Italic Text', True, True);
  AddItem('ALIGNJUSTIFY', SCtrl+'+J', 'Justifies the paragraph between the margins', True, True);
  AddItem('JUSTIFY', SCtrl+'+J', 'Justifies the paragraph between the margins', True, False);
  AddItem('MARKASREAD', '', 'Marks the selected messages as read', True, True);
  AddItem('MARKASUNREAD', '', 'Marks the selected messages as unread', True, True);
  AddItem('MOVETOFOLDER', '', 'Moves selected items to another folder', True, True);
  AddItem('NEW', SCtrl+'+N', 'Create a new document', True, True);
  AddItem('NEWFOLDER', '', 'Create a new folder', True, True);
  AddItem('NEWMAIL', '', 'Create a new email message', True, True);
  AddItem('NEWMAILMESSAGE', '', 'Create a new email message', True, False);
  AddItem('NEWNOTE', '', 'Create a new note', True, True);
  AddItem('NEWTASK', '', 'Create a new task', True, True);
  AddItem('NOTE', '', 'Create a new note', True, False);
  AddItem('NUMBERING', '', 'Inserts a numbering on this line', True, True);
  AddItem('OPEN', SCtrl+'+O', 'Opens an existing document from a file', True, True);
  AddItem('PASTE', SCtrl+'+V;'+SShift+'+'+SIns, 'Inserts the clipboard contents at the insertion point', True, True);
  AddItem('PRINT', SCtrl+'+P', 'Print the active document', True, True);
  AddItem('PRINTPREVIEW', '', 'Display full pages', True, True);
  AddItem('PROPERTIES', '', 'Display properties for the active document', True, True);
  AddItem('REDO', SCtrl+'+'+SShift+'+Z', 'Redoes the last action that was undone', True, True);
  AddItem('REFRESH', '', 'Updates the display to reflect any changes', True, True);
  AddItem('RELOAD', '', 'Reloads the content of the current page', True, False);
  AddItem('REPEAT', '', 'Repeats the last action', True, True);
  AddItem('REPLACE', SCtrl+'+H', 'Replaces specific text with different text', True, True);
  AddItem('SAVE', SCtrl+'+S;'+SShift+'+F12;'+SAlt+'+'+SShift+'+F2', 'Saves the active document', True, True);
  AddItem('SAVEALL', '', 'Saves all opened documents', True, True);
  AddItem('SEARCHTHEWEB', '', 'Searches the Word Wide Web for the specified resource', True, True);
  AddItem('SENDMAIL', '', 'Compose and send new mail message', True, True);
  AddItem('SETBOOKMARK', '', 'Insert a bookmark at the cursor', True, True);
  AddItem('SETDATE', '', 'Set today''s date', True, True);
  AddItem('SETFONT', '', 'View or edit the character attributes of the selected text', True, True);
  AddItem('SETTIME', '', 'Set current time', True, True);
  AddItem('SORT', '', 'Arranges items in the window', True, True);
  AddItem('SPELLCHECK', 'F7', 'Checks the spelling in the active document', True, True);
  AddItem('CHECKSPELLING', 'F7', 'Checks the spelling in the active document', True, False);
  AddItem('SPLITWINDOW', '', 'Splits the active document window horizontally', True, True);
  AddItem('TASK', '', 'Create a new task', True, False);
  AddItem('TILE', '', 'Arranges the windows as nonoverlapping tiles', True, True);
  AddItem('TIME', '', 'Inserts current time', True, False);
  AddItem('UNDERLINE', SCtrl+'+U', 'Underlined text', True, True);
  AddItem('UNDO', SCtrl+'+Z', 'Reverses the last action', True, True);
  AddItem('UNINDENT', '', 'Decrease indent', True, False);
  AddItem('WHATSTHIS', SShift+'+F1', 'Shows help about the selected element', True, True);
  AddItem('PARAGRAPH', '', 'View or edit the paragraph attributes of the selected text', True, True);
  AddItem('MOVEUP', '', 'Moves the selection up', True, True);
  AddItem('MOVEDOWN', '', 'Moves the selection down', True, True);
  AddItem('MOVELEFT', '', 'Moves the selection left', True, True);
  AddItem('MOVERIGHT', '', 'Moves the selection right', True, True);
  AddItem('UPONELEVEL', '', 'Up one level', True, True);
  AddItem('RULER', '', 'Shows or hides the ruler', True, True);
  AddItem('DELETE_B', '', 'Erases the selection', True, True);
  AddItem('NEWFOLDER_B', '', 'Create a new folder', True, True);
  AddItem('CREATE', '', 'Create a new document', True, True);
  AddItem('CREATE_B', '', 'Create a new document', True, True);
  AddItem('BACKWARD', '', 'Return to the previous page in the hyperlink history list', True, True);
  AddItem('BACK', '', 'Return to the previous page in the hyperlink history list', True, False);
  AddItem('FORWARD', '', 'Go to the next page in the hyperlink history list', True, True);

  // other items
  AddItem('CLEAR', SDel+';'+SShift+'+'+SDel, 'Erases the selection', False, False);
  AddItem('SELECTALL', SCtrl+'+A', 'Select the entire document', False, False);
  AddItem('GOTO', SCtrl+'+G', 'Moves the cursor to the specified line', False, False);
  AddItem('PAGESETUP', '', 'Change the printing options', False, False);
  AddItem('PRINTERSETUP', '', 'Change the printer and printing options', False, False);
  AddItem('SAVEAS', 'F12;'+SCtrl+'+'+SShift+'+S', 'Saves a copy of the document in a separate file', False, False);
  AddItem('PASTELINK', '', 'Insert Clipboard contents and a link to its source', False, False);
  AddItem('PASTESPECIAL', '', 'Insert Clipboard contents with options', False, False);
  AddItem('NEWWINDOW', '', 'Open another window for the active document', False, False);
  AddItem('ARRANGEICONS', '', 'Arrange icons at the bottom of the window', False, False);
  AddItem('EDIT', '', 'Contains commands for the clipboard, finding text, and editing links', False, False);
  AddItem('VIEW', '', 'Contains commands for controlling the display of your document', False, False);
  AddItem('FILE', '', 'Contains commands for saving documents and opening saved documents', False, False);
  AddItem('SEARCH', '', 'Contains commands for searching through your document', False, False);
  AddItem('TOOLS', '', 'Contains additional commands for working with external tools', False, False);
  AddItem('WINDOW', '', 'Contains commands for controlling position of windows of your application', False, False);
  AddItem('HELP', '', 'Contains commands for solving problem situations', False, False);

  // system icons
  AddItem('SYSTEMRESTORE', '', 'Restore the window to normal size', True, False);
  AddItem('SYSTEMMOVE', '', 'Change the window position', True, False);
  AddItem('SYSTEMSIZE', '', 'Change the window size', True, False);
  AddItem('SYSTEMMINIMIZE', '', 'Reduce the window to an icon', True, False);
  AddItem('SYSTEMMAXIMIZE', '', 'Enlarge the window to full size', True, False);
  AddItem('SYSTEMCLOSE', '', 'Close the active window and prompts to save the pages', True, False);

  // create image list
  Images:= ImageList_LoadBitmap(HInstance, 'BMP_AM2000_MENUITEMCACHE', 16, 1, ColorToRGB(clFuchsia));

end;

destructor T_AM2000_MenuItemCache.Destroy;
var
  I: Integer;
begin
  for I:= 0 to Count -1 do
    T_AM2000_MenuItemCacheItem(Objects[I]).Free;

  // destroy image list
  ImageList_Destroy(Images);

  inherited;
end;

function T_AM2000_MenuItemCache.AddItem(ACaption, AShortCuts, AHint: String;
  ABitmap, ADefault: Boolean): T_AM2000_MenuItemCacheItem;
var
  Index: Integer;
begin
  ACaption:= GetStandardCaption(ACaption);
  if not Find(ACaption, Index) then begin
    Result:= T_AM2000_MenuItemCacheItem.Create;
    AddObject(ACaption, Result);
  end
  else
    Result:= T_AM2000_MenuItemCacheItem(Objects[Index]);

  Result.ShortCuts:= AShortCuts;
  Result.Hint:=      AHint;
  Result.HasBitmap:= ABitmap;
  Result.IsDefault:= ADefault;
end;

function T_AM2000_MenuItemCache.GetItem(const Caption: String): T_AM2000_MenuItemCacheItem;
var
  Index: Integer;
  S: String;
begin
  S:= GetStandardCaption(Caption);
  if Find(S, Index)
  then Result:= T_AM2000_MenuItemCacheItem(Objects[Index])
  else Result:= nil;
end;

procedure T_AM2000_MenuItemCache.Draw(Canvas: TCanvas; X, Y: Integer; Caption: String; Color1, Color2: TColor; Enabled: Boolean);
var
  Index: Integer;
begin
  if not Find(GetStandardCaption(Caption), Index)
  then Exit;

  if Enabled
  then ImageList_Draw(Images, Index, Canvas.Handle, X, Y, ild_Transparent)
  else ImgDisabledBlt(Canvas, X, Y, Images, Index, Color1, Color2);
end;

procedure T_AM2000_MenuItemCache.DrawIndex(Canvas: TCanvas; X, Y,
  ImageIndex: Integer; Color1, Color2: TColor; Enabled: Boolean);
begin
  if (ImageIndex < 0) or (ImageIndex >= Count)
  then Exit;

  if Enabled
  then ImageList_Draw(Images, ImageIndex, Canvas.Handle, X, Y, ild_Transparent)
  else ImgDisabledBlt(Canvas, X, Y, Images, ImageIndex, Color1, Color2);
end;

function T_AM2000_MenuItemCache.IsDefault(Caption: String): Boolean;
var
  Index: Integer;
begin
  Result:= Find(GetStandardCaption(Caption), Index);
end;


initialization
  MenuItemCache:= T_AM2000_MenuItemCache.Create;

finalization
  MenuItemCache.Free;
  MenuItemCache:= nil;

end.
